@@ -185,10 +185,21 @@ class TimeSeries(object):
 
         if isinstance(values, (list, tuple,)) and values:
             values = np.asarray(values)
+        if get_type(values) == np.int64:
+            # Create time as int32 because DAP does not support int64 until DAP4.
+            values = values.astype(np.int32)
+
         if isinstance(times, (list, tuple,)) and times:
             times = np.asarray(times)
+        if get_type(times) == np.int64:
+            # Create time as int32 because DAP does not support int64 until DAP4.
+            times = times.astype(np.int32)
+
         if isinstance(verticals, (list, tuple,)) and verticals:
             verticals = np.asarray(verticals)
+        if get_type(verticals) == np.int64:
+            # Create time as int32 because DAP does not support int64 until DAP4.
+            verticals = verticals.astype(np.int32)
 
         # Set vertical datum on the CRS variable
         if sensor_vertical_datum is not None:
@@ -265,7 +276,7 @@ class TimeSeries(object):
                         # Create metadata variable for the sensor_depth
                         if nc.variables.get('sensor_depth') is None:
                             logger.info("Setting the special case 'sensor_depth' metadata variable")
-                            inst_depth = nc.createVariable('sensor_depth', 'f4')
+                            inst_depth = nc.createVariable('sensor_depth', get_type(verticals))
                             inst_depth.units = 'm'
                             inst_depth.standard_name = 'surface_altitude'
                             inst_depth.positive = self.vertical_positive
@@ -346,6 +357,9 @@ class TimeSeries(object):
 
         if isinstance(times, (list, tuple,)):
             times = np.asarray(times)
+        if get_type(times) == np.int64:
+            # Create time as int32 because DAP does not support int64 until DAP4.
+            times = times.astype(np.int32)
 
         # If nothing is passed in, set to the vertical_fill value.
         if not isinstance(verticals, np.ndarray) and not verticals:
@@ -357,6 +371,9 @@ class TimeSeries(object):
         elif isinstance(verticals, np.ndarray):
             self.vertical_fill = verticals.dtype.type(self.vertical_fill)
             verticals = np.ma.masked_values(verticals, self.vertical_fill)
+        if get_type(verticals) == np.int64:
+            # Create time as int32 because DAP does not support int64 until DAP4.
+            verticals = verticals.astype(np.int32)
 
         self.time_indexes = np.argsort(times)
         unique_times = times[self.time_indexes]
@@ -501,9 +518,15 @@ def get_dataframe_from_variable(nc, data_var):
             logger.warning("Converting depths to positive down before returning the DataFrame")
             depths = depths * -1
 
+    # https://github.com/numpy/numpy/issues/4595
+    # We can't call astype on a MaskedConstant
+    if (isinstance(depths, np.ma.core.MaskedConstant) or
+       (hasattr(depths, 'mask') and depths.mask.all())):
+        depths = np.asarray([np.nan] * len(times)).flatten()
+
     df = pd.DataFrame({ 'time':   times,
                         'value':  values.astype(data_var.dtype),
-                        'unit':   data_var.units,
+                        'unit':   data_var.units if hasattr(data_var, 'units') else np.nan,
                         'depth':  depths.astype(depth_type) })
 
     df.set_index([pd.DatetimeIndex(df['time']), pd.Float64Index(df['depth'])], inplace=True)
