@@ -60,6 +60,7 @@ class TestTimeSeries(unittest.TestCase):
             nc.geospatial_vertical_max
 
         assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
         assert nc.variables.get('temperature').size == len(times)
         assert (nc.variables.get('temperature')[:] == np.asarray(values)).all()
 
@@ -98,6 +99,7 @@ class TestTimeSeries(unittest.TestCase):
             nc.geospatial_vertical_max
 
         assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
         assert nc.variables.get('temperature').size == len(times)
         assert (nc.variables.get('temperature')[:] == np.asarray(values[0:6])).all()
 
@@ -133,8 +135,10 @@ class TestTimeSeries(unittest.TestCase):
         self.assertEqual(nc.geospatial_vertical_max, 2)
 
         assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
         assert nc.variables.get('z').size == len(verticals)
         assert nc.variables.get('z').positive == 'down'
+        assert nc.variables.get('z')[:].dtype == np.int32
         assert nc.variables.get('temperature').size == len(times) * len(verticals)
         assert (nc.variables.get('temperature')[:] == values.reshape((len(times), len(verticals)))).all()
 
@@ -166,8 +170,10 @@ class TestTimeSeries(unittest.TestCase):
         self.assertEqual(nc.geospatial_vertical_max, 2)
 
         assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
         assert nc.variables.get('height').size == len(verticals)
         assert nc.variables.get('height').positive == 'up'
+        assert nc.variables.get('height')[:].dtype == np.int32
         assert nc.variables.get('temperature').size == len(times) * len(verticals)
         assert (nc.variables.get('temperature')[:] == values.reshape((len(times), len(verticals)))).all()
 
@@ -204,6 +210,7 @@ class TestTimeSeries(unittest.TestCase):
         self.assertEqual(nc.geospatial_vertical_max, 2)
 
         assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
         assert nc.variables.get('z').size == len(verticals)
         assert nc.variables.get('temperature').size == len(times) * len(verticals)
         assert (nc.variables.get('temperature')[:] == np.repeat([20, 21, 22, 23, 24, 25], len(verticals)).reshape((len(times), len(verticals)))).all()
@@ -233,7 +240,9 @@ class TestTimeSeries(unittest.TestCase):
         self.assertEqual(nc.geospatial_vertical_max, 1)
 
         assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
         assert nc.variables.get('z').size == len(list(set(verticals)))
+        assert nc.variables.get('z')[:].dtype == np.int32
         assert nc.variables.get('temperature').size == len(times) * len(list(set(verticals)))
 
         assert (nc.variables.get('temperature')[:] == values.reshape((len(times), 2))).all()
@@ -263,6 +272,7 @@ class TestTimeSeries(unittest.TestCase):
         self.assertEqual(nc.geospatial_vertical_max, 2)
 
         assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
         assert nc.variables.get('z').size == len(verticals)
         assert nc.variables.get('temperature').size == len(times) * len(verticals)
         assert (nc.variables.get('temperature')[:] == values.reshape((len(times), len(verticals)))).all()
@@ -293,26 +303,28 @@ class TestTimeSeries(unittest.TestCase):
         self.assertEqual(nc.geospatial_vertical_max, 0)
 
         assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
         assert nc.variables.get('z').size == len(verticals)
+        assert nc.variables.get('z')[:].dtype == np.float64
         assert nc.variables.get('temperature').size == len(times) * len(verticals)
 
         assert nc.variables.get('temperature')[:][0][1] == 20
-        assert nc.variables.get('temperature')[:].mask[0][0] == True
+        assert nc.variables.get('temperature')[:].mask[0][0] == True  # noqa
 
         assert nc.variables.get('temperature')[:][1][1] == 21
-        assert nc.variables.get('temperature')[:].mask[1][0] == True
+        assert nc.variables.get('temperature')[:].mask[1][0] == True  # noqa
 
         assert nc.variables.get('temperature')[:][2][1] == 22
-        assert nc.variables.get('temperature')[:].mask[2][0] == True
+        assert nc.variables.get('temperature')[:].mask[2][0] == True  # noqa
 
         assert nc.variables.get('temperature')[:][3][1] == 23
-        assert nc.variables.get('temperature')[:].mask[3][0] == True
+        assert nc.variables.get('temperature')[:].mask[3][0] == True  # noqa
 
         assert nc.variables.get('temperature')[:][4][1] == 24
-        assert nc.variables.get('temperature')[:].mask[4][0] == True
+        assert nc.variables.get('temperature')[:].mask[4][0] == True  # noqa
 
         assert nc.variables.get('temperature')[:][5][1] == 25
-        assert nc.variables.get('temperature')[:].mask[5][0] == True
+        assert nc.variables.get('temperature')[:].mask[5][0] == True  # noqa
 
         assert (nc.variables.get('temperature')[:] == np.asarray(values).reshape((len(times), len(verticals)))).all()
 
@@ -341,7 +353,9 @@ class TestTimeSeries(unittest.TestCase):
         self.assertEqual(nc.geospatial_vertical_max, 50)
 
         assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
         assert nc.variables.get('z').size == len(verticals)
+        assert nc.variables.get('z')[:].dtype == np.int32
         assert nc.variables.get('temperature').size == len(times) * len(verticals)
 
         assert nc.variables.get('temperature')[:][0][0] == 25
@@ -384,7 +398,9 @@ class TestTimeSeries(unittest.TestCase):
         self.assertEqual(nc.geospatial_vertical_max, 2)
 
         assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
         assert nc.variables.get('z').size == len(verticals)
+        assert nc.variables.get('z')[:].dtype == np.int32
         assert nc.variables.get('temperature').size == len(times) * len(verticals)
         assert nc.variables.get('sensor_depth') is not None
         assert nc.variables.get('bottom_temperature').size == len(times)
@@ -423,12 +439,118 @@ class TestTimeSeries(unittest.TestCase):
         self.assertEqual(nc.geospatial_vertical_max, 2)
 
         assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
         assert nc.variables.get('z').size == len(verticals)
+        assert nc.variables.get('z')[:].dtype == np.int32
         assert nc.variables.get('temperature').size == len(times) * len(verticals)
         assert (nc.variables.get('temperature')[:] == values.reshape((len(times), len(verticals)))).all()
         assert (nc.variables.get('salinity')[:] == values.reshape((len(times), len(verticals)))).all()
         assert nc.variables.get('dissolved_oxygen')[:].mask.all()
 
+    def test_extracting_dataframe_all_masked_heights(self):
+        filename = 'test_extracting_dataframe_all_masked_heights.nc'
+        times = [0, 1000, 2000, 3000, 4000, 5000]
+        verticals = [-9999.9]
+        ts = TimeSeries(output_directory=self.output_directory,
+                        latitude=self.latitude,
+                        longitude=self.longitude,
+                        station_name=self.station_name,
+                        global_attributes=self.global_attributes,
+                        output_filename=filename,
+                        times=times,
+                        verticals=verticals,
+                        vertical_fill=-9999.9)
+
+        values = np.repeat([20, 21, 22, 23, 24, 25], len(verticals))
+        attrs = dict(standard_name='sea_water_temperature')
+        ts.add_variable('temperature', values=values, attributes=attrs)
+
+        nc = netCDF4.Dataset(os.path.join(self.output_directory, filename))
+        assert nc is not None
+
+        assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
+        assert nc.variables.get('z').size == len(verticals)
+        assert nc.variables.get('z')[:].dtype == np.float64
+        assert nc.variables.get('z')[:].size == 1
+        assert nc.variables.get('z')[:].mask == True  # noqa
+        assert nc.variables.get('temperature').size == len(times) * len(verticals)
+
+        df = get_dataframe_from_variable(nc, nc.variables.get('temperature'))
+        assert df['depth'].dropna().empty
+
+    def test_extracting_dataframe_some_masked_heights(self):
+        filename = 'test_extracting_dataframe_some_masked_heights.nc'
+        times = [0, 1000, 2000, 3000, 4000, 5000]
+        verticals = [-9999.9, 7.8, 7.9]
+        ts = TimeSeries(output_directory=self.output_directory,
+                        latitude=self.latitude,
+                        longitude=self.longitude,
+                        station_name=self.station_name,
+                        global_attributes=self.global_attributes,
+                        output_filename=filename,
+                        times=times,
+                        verticals=verticals,
+                        vertical_fill=-9999.9)
+
+        values = np.repeat([20, 21, 22, 23, 24, 25], len(verticals))
+        attrs = dict(standard_name='sea_water_temperature')
+        ts.add_variable('temperature', values=values, attributes=attrs)
+
+        nc = netCDF4.Dataset(os.path.join(self.output_directory, filename))
+        assert nc is not None
+
+        assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
+        assert nc.variables.get('z').size == len(verticals)
+        assert nc.variables.get('z')[:].dtype == np.float64
+        assert np.allclose(nc.variables.get('z')[:], np.ma.array([np.nan, 7.8, 7.9], mask=[1, 0, 0]))
+        assert nc.variables.get('temperature').size == len(times) * len(verticals)
+
+        df = get_dataframe_from_variable(nc, nc.variables.get('temperature'))
+        assert not df['depth'].dropna().empty
+
+    def test_extracting_dataframe_ordered_masked_heights(self):
+        filename = 'test_extracting_dataframe_ordered_masked_heights.nc'
+        times = [0, 1000, 2000, 3000, 4000, 5000]
+        verticals = [np.nan, 7.8]
+        ts = TimeSeries(output_directory=self.output_directory,
+                        latitude=self.latitude,
+                        longitude=self.longitude,
+                        station_name=self.station_name,
+                        global_attributes=self.global_attributes,
+                        output_filename=filename,
+                        times=times,
+                        verticals=verticals,
+                        vertical_fill=np.nan)
+
+        values = np.asarray([[20, 21], [22, 23], [24, 25], [30, 31], [32, 33], [34, 35]])
+        attrs = dict(standard_name='sea_water_temperature')
+        ts.add_variable('temperature', values=values, attributes=attrs)
+
+        nc = netCDF4.Dataset(os.path.join(self.output_directory, filename))
+        assert nc is not None
+
+        assert nc.variables.get('time').size == len(times)
+        assert nc.variables.get('time')[:].dtype == np.int32
+        assert nc.variables.get('z').size == len(verticals)
+        assert nc.variables.get('z')[:].dtype == np.float64
+
+        # The height order is sorted!
+        assert np.allclose(nc.variables.get('z')[:], np.ma.array([7.8, np.nan], mask=[0, 1]))
+        assert nc.variables.get('temperature').size == len(times) * len(verticals)
+
+        # Be sure the values are re-arranged because the height order is sorted!
+        assert np.isclose(nc.variables.get('temperature')[:][0][0], 21)
+        assert np.isclose(nc.variables.get('temperature')[:][1][0], 23)
+        assert np.isclose(nc.variables.get('temperature')[:][2][0], 25)
+        assert np.isclose(nc.variables.get('temperature')[:][3][0], 31)
+        assert np.isclose(nc.variables.get('temperature')[:][4][0], 33)
+        assert np.isclose(nc.variables.get('temperature')[:][5][0], 35)
+
+        df = get_dataframe_from_variable(nc, nc.variables.get('temperature'))
+        assert not df['depth'].dropna().empty
+
 
 class TestTimeseriesTimeBounds(unittest.TestCase):
 
@@ -531,6 +653,4 @@ class TestDataFrameFromVariable(unittest.TestCase):
         ncvar1 = ncd1.variables['soil_moisture_percent']
         df1 = get_dataframe_from_variable(ncd1, ncvar1)
 
-        logger.info(df1.depth.unique())
-
         assert np.allclose(df1.depth.unique(), np.asarray([-0.0508, -0.2032, -0.508]))
